unit ButtonEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,Buttons,AboutClass;

type
  TNumGlyphs=1..4;
  TButtonEdit = class(TEdit)
  private
    { Private declarations }
    FAbout :TMyAboutBoxObject;
    FButton: TSpeedButton;
    FButtonCaption:string;
    FButtonHint:string;
    FButtonGraph:TBitmap;
    FButtonFlat:boolean;
    FButtonEnabled:boolean;
    FButtonFont:TFont;
    FOnButtonClick: TNotifyEvent;
    FButtonWidth: integer;
    FButtonNumGlyphs: TNumGlyphs;
    procedure WMSize( var Msg: TWMSize ); message wm_Size;
    procedure SetEditRect;
    procedure SetButtonCaption(const Value: String);
    procedure SetButtonGraph(const Value: TBitmap);
    procedure SetButtonHint(const Value: String);
    procedure SetButtonFlat(const Value:boolean);
    procedure SetButtonFont(const Value:TFont);
    procedure SetButtonEnabled(const Value: Boolean);
    procedure SetButtonWidth(const Value: integer);
    procedure SetFButtonNumGlyphs(const Value: TNumGlyphs);
  protected
    { Protected declarations }
    procedure CreateParams( var Params: TCreateParams ); override;
    procedure CreateWnd; override;
    procedure ButtonClickHandler( Sender: TObject ); virtual;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
  published
    { Published declarations }
    property About:TMyAboutBoxObject read FAbout write FAbout;
    property ButtonCaption:String read FButtonCaption write SetButtonCaption;
    property ButtonHint:String read FButtonHint write SetButtonHint;
    property ButtonGraph:TBitmap read FButtonGraph write SetButtonGraph;
    property ButtonFlat:boolean read FButtonFlat write SetButtonFlat;
    property ButtonFont:Tfont read FButtonFont write SetButtonFont;
    property ButtonEnabled:Boolean read FButtonEnabled write SetButtonEnabled;
    property ButtonWidth:integer read FButtonWidth write SetButtonWidth;
    property ButtonNumGlyphs:TNumGlyphs read FButtonNumGlyphs write SetFButtonNumGlyphs Default 2;
    property OnButtonClick: TNotifyEvent read FOnButtonClick write FOnButtonClick;
  end;


implementation

{ TButtonEdit }

procedure TButtonEdit.ButtonClickHandler(Sender: TObject);
begin
  if Assigned( FOnButtonClick ) then
  FOnButtonClick( Self );
end;

constructor TButtonEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [ csSetCaption ];
  FButtonFont := TFont.Create;
  FButtonGraph := TBitmap.Create;
  FButton := TSpeedButton.Create(self);
  FButton.Parent := Self;
  FButton.Font.Assign(Font);
  FButtonFont.Assign(Font);
  FButton.NumGlyphs := 2;
  FButton.Width := FButton.Height;
  FButtonWidth := FButton.Height;
  FButtonWidth := 17;
  FButton.Cursor := crArrow;
  FButton.ShowHint := True;
  FButton.OnClick := ButtonClickHandler;
  FButton.Visible := True;
end;

procedure TButtonEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams( Params );
  Params.Style := Params.Style or ws_ClipChildren;
end;

procedure TButtonEdit.CreateWnd;
begin
  inherited CreateWnd;
  SetEditRect;
end;

destructor TButtonEdit.Destroy;
begin
  FButton.Free;
  FButtonFont.Free;
  FButtonGraph.Free;
  inherited Destroy;
end;

procedure TButtonEdit.SetButtonCaption(const Value: String);
begin
  FButtonCaption := Value;
  FButton.Caption := Value;
  SendMessage(Handle,WM_SIZE,0,0);
  Invalidate;
end;

procedure TButtonEdit.SetButtonEnabled(const Value: Boolean);
begin
  FButtonEnabled := Value;
  FButton.Enabled := Value;
  SendMessage(Handle,WM_SIZE,0,0);
  Invalidate;
end;

procedure TButtonEdit.SetButtonFlat(const Value: boolean);
begin
  FButtonFlat := Value;
  FButton.Flat := Value;
  SendMessage(Handle,WM_SIZE,0,0);
  Invalidate;
end;

procedure TButtonEdit.SetButtonFont(const Value: TFont);
begin
  FButtonFont.Assign(Value);
  FButton.Font.Assign (Value);
  SendMessage(Handle,WM_SIZE,0,0);
  Invalidate;
end;

procedure TButtonEdit.SetButtonGraph(const Value: TBitmap);
begin
  FButton.NumGlyphs := FButtonNumGlyphs;
  FButtonGraph.Assign(Value);
  FButton.Glyph.Assign(Value);
  SendMessage(Handle,WM_SIZE,0,0);
  Invalidate;
end;

procedure TButtonEdit.SetButtonHint(const Value: String);
begin
  FButtonHint := Value;
  FButton.Hint := Value;
  SendMessage(Handle,WM_SIZE,0,0);
  Invalidate;
end;


procedure TButtonEdit.SetButtonWidth(const Value: integer);
begin
  FButtonWidth := Value;
  FButton.Width := Value;
  SendMessage(Handle,WM_SIZE,0,0);
  Invalidate;
end;

procedure TButtonEdit.SetEditRect;
begin
  SendMessage( Handle, em_SetMargins, ec_LeftMargin, 0 );
  SendMessage( Handle, em_SetMargins, ec_RightMargin, MakeLong( 0, FButton.Width + 2 ) );
end;

procedure TButtonEdit.SetFButtonNumGlyphs(const Value: TNumGlyphs);
begin
  FButtonNumGlyphs := Value;
  FButton.NumGlyphs := Value;
  SendMessage(Handle,WM_SIZE,0,0);
  Invalidate;
end;

procedure TButtonEdit.WMSize(var Msg: TWMSize);
begin
  if self.BorderStyle <> bsNone then begin
    FButton.Height := Height - 4;
    FButton.Width := FButtonWidth;
    FButton.SetBounds(Width-FButtonWidth-4,0 ,FButtonWidth,FButton.Height);
  end
  else
    begin
      FButton.Height := Height;
      FButton.Width := FButtonWidth;
      FButton.SetBounds(Width-FButtonWidth,0 ,FButtonWidth,FButton.Height);
    end;
  if FButtonGraph <> nil then begin
    FButton.NumGlyphs := FButtonNumGlyphs;
    FButton.Glyph.Assign(FButtonGraph);
  end;
  SetEditRect;
  Invalidate;
  end;
end.
