unit FileComboBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,FileCtrl,ExtCtrls,ShellAPI,AboutClass;

type
  TFileAttrs = set of (ReadOnly,Hidden,SysFile,Archive);
  TFileInfo = record
    FileName:String;
    FileSize:integer;
    FileDate:TDateTime;
    FileAttr:TFileAttrs;
  end;
  TFileComboBox = class(TComboBox)
  private
  // user declaration
    FAbout :TMyAboutBoxObject;
  // IDE generate
    FPath: String;
    FDirectoryListBox: TDirectoryListBox;
    FAttrHint: boolean;
    procedure SetPath(const Value: String);
    procedure FillItems(FPath:String);
    procedure SetDirectoryListBox(const Value: TDirectoryListBox);
    procedure SetAtrrHint(const Value: boolean);
    function AttrToFileAttr(Attr:integer):TFileAttrs;
    function AttrToStr(FileAttr:TFileAttrs):String;
    { Private declarations }
  protected
    { Protected declarations }
    procedure Change;override;
  public
    { Public declarations }
    FileInfoArray:array of TFileInfo;
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;

  published
    { Published declarations }
    property About:TMyAboutBoxObject read FAbout write FAbout;
    property Path:String read FPath write SetPath;
    property DirectoryListBox:TDirectoryListBox
                read FDirectoryListBox
                write SetDirectoryListBox;
    property UseFileHint:boolean
                read FAttrHint
                write SetAtrrHint;
  end;


implementation

{ TFileComboBox }

function TFileComboBox.AttrToFileAttr(Attr: integer): TFileAttrs;
begin
  Result := [];
  if (Attr and faHidden <> 0) then Result := Result + [Hidden];
  if (Attr and faReadOnly <> 0) then Result := Result + [ReadOnly];
  if (Attr and faSysFile <> 0) then Result := Result + [SysFile];
  if (Attr and faArchive <> 0) then Result := Result + [Archive];
end;

function TFileComboBox.AttrToStr(FileAttr: TFileAttrs): String;
begin
  Result := '';
  if Hidden in FileAttr then Result := Result + ' Hidden ';
  if ReadOnly in FileAttr then Result := Result + ' ReadOnly ';
  if SysFile in FileAttr then Result := Result + ' SysFile ';
  if Archive in FileAttr then Result := Result + ' Archive ';
end;

procedure TFileComboBox.Change;
begin
  inherited;
  if ItemIndex = -1 then Exit;
  if FAttrHint then begin
    Hint := ' '+FileInfoArray[ItemIndex].FileName+#13+
            ' '+IntToStr(FileInfoArray[ItemIndex].FileSize)+#13+
            ' '+DateTimeToStr(FileInfoArray[ItemIndex].FileDate)+' '+#13+
            AttrToStr(FileInfoArray[ItemIndex].FileAttr);
  end;
end;

constructor TFileComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ShowHint := True;
  Style := csOwnerDrawFixed;
  FAttrHint := True;
  Width := 145;
  Height := 22;
end;

destructor TFileComboBox.Destroy;
begin
  inherited Destroy;
end;

procedure TFileComboBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
var
  Info : TSHFileInfo;
  xFileName : String;
begin
  if FPath[Length(FPath)]<>'\' then xFilename:=FPath+'\'+Items[Index]
                                         else xFilename:=FPath+Items[Index];
  with Canvas do
  begin
    FillRect(Rect);
    FillChar(Info,SizeOF(TSHFileInfo),0);
    with TImageList.Create(self) do
    try
      Handle:=SHGetFileInfo(PChar(XFileName),0,Info,
        SizeOf(TSHFileInfo),SHGFI_SYSICONINDEX or SHGFI_SMALLICON or SHGFI_OPENICON);
        ShareImages:=True;
        Draw(Canvas,Rect.Left,Rect.Top,Info.iIcon);
    finally
       Free;
    end;
    TextOut(Rect.Left+ 22, Rect.Top, Items[Index]);
  end;
end;

procedure TFileComboBox.FillItems(FPath: String);
var
  sr:TSearchRec;
  i:integer;
begin
  Items.Clear;
  if Pos('*.*',FPath) = 0 then begin
    if Pos('\',FPath) = length(FPath) then
      FPath := FPath + '*.*'
    else
      FPath := FPath + '\*.*';
  end;
  if FindFirst(FPath,faReadOnly+faHidden+faSysFile+faArchive,sr) = 0 then begin
    Items.Add(sr.Name);
    SetLength(FileInfoArray,1);
    FileInfoArray[0].FileName := sr.Name;
    FileInfoArray[0].FileSize := sr.Size;
    FileInfoArray[0].FileDate := FileDateToDateTime(sr.Time);
    FileInfoArray[0].FileAttr := AttrToFileAttr(sr.Attr);
    i := 1;
    while FindNext(sr) = 0 do
    begin
      Items.Add(sr.Name);
      SetLength(FileInfoArray,i + 1);
      FileInfoArray[i].FileName := sr.Name;
      FileInfoArray[i].FileSize := sr.Size;
      FileInfoArray[i].FileDate := FileDateToDateTime(sr.Time);
      FileInfoArray[i].FileAttr := AttrToFileAttr(sr.Attr);
      Inc(i);
    end;
  end;
  FindClose(sr);
end;

procedure TFileComboBox.SetAtrrHint(const Value: boolean);
begin
  FAttrHint := Value;
end;

procedure TFileComboBox.SetDirectoryListBox(
  const Value: TDirectoryListBox);
begin
  FDirectoryListBox := Value;
  if FDirectoryListBox <> nil then
    FPath := FDirectoryListBox.Directory
  else
    FPath := GetCurrentDir;
  FillItems(FPath);
end;

procedure TFileComboBox.SetPath(const Value: String);
begin
  FPath := Value;
  FillItems(FPath);
end;

end.
