unit WyColorComboBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,AboutClass;

type
  TWyColorComboBox = class(TComboBox)
  private
    FAbout :TMyAboutBoxObject;
    FCustomColor:TColor;
    FNoText: boolean;
    procedure SetNoText(const Value: boolean);
    function GetSelectedColorName: String;
    function GetSelectedColor: TColor;
    { Private declarations }
  protected
    { Protected declarations }
    procedure Change; override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    property SelectedColorName : String read GetSelectedColorName;
    property SelectedColor : TColor read GetSelectedColor;
  published
    { Published declarations }
    property About:TMyAboutBoxObject read FAbout write FAbout;
    property NoText:boolean read FNoText write SetNoText;
    property ItemIndex;
  end;
  TWyColorEntry = record
    Value: TColor;
    Name: string;
  end;
const
  Colors: array[-1..41] of TWyColorEntry = (
    (Value: clNone; Name: 'None'),
    (Value: clBlack; Name: 'Black'),
    (Value: clMaroon; Name: 'Maroon'),
    (Value: clGreen; Name: 'Green'),
    (Value: clOlive; Name: 'Olive'),
    (Value: clNavy; Name: 'Navy'),
    (Value: clPurple; Name: 'Purple'),
    (Value: clTeal; Name: 'Teal'),
    (Value: clGray; Name: 'Gray'),
    (Value: clSilver; Name: 'Silver'),
    (Value: clRed; Name: 'Red'),
    (Value: clLime; Name: 'Lime'),
    (Value: clYellow; Name: 'Yellow'),
    (Value: clBlue; Name: 'Blue'),
    (Value: clFuchsia; Name: 'Fuchsia'),
    (Value: clAqua; Name: 'Aqua'),
    (Value: clWhite; Name: 'White'),
    (Value: clScrollBar; Name: 'ScrollBar'),
    (Value: clBackground; Name: 'Background'),
    (Value: clActiveCaption; Name: 'ActiveCaption'),
    (Value: clInactiveCaption; Name: 'InactiveCaption'),
    (Value: clMenu; Name: 'Menu'),
    (Value: clWindow; Name: 'Window'),
    (Value: clWindowFrame; Name: 'WindowFrame'),
    (Value: clMenuText; Name: 'MenuText'),
    (Value: clWindowText; Name: 'WindowText'),
    (Value: clCaptionText; Name: 'CaptionText'),
    (Value: clActiveBorder; Name: 'ActiveBorder'),
    (Value: clInactiveBorder; Name: 'InactiveBorder'),
    (Value: clAppWorkSpace; Name: 'AppWorkSpace'),
    (Value: clHighlight; Name: 'Highlight'),
    (Value: clHighlightText; Name: 'HighlightText'),
    (Value: clBtnFace; Name: 'BtnFace'),
    (Value: clBtnShadow; Name: 'BtnShadow'),
    (Value: clGrayText; Name: 'GrayText'),
    (Value: clBtnText; Name: 'BtnText'),
    (Value: clInactiveCaptionText; Name: 'InactiveCaptionText'),
    (Value: clBtnHighlight; Name: 'BtnHighlight'),
    (Value: cl3DDkShadow; Name: '3DDkShadow'),
    (Value: cl3DLight; Name: '3DLight'),
    (Value: clInfoText; Name: 'InfoText'),
    (Value: clInfoBk; Name: 'InfoBk'),
    (Value: clWhite; Name: 'Custom'));

implementation

{ TWyColorComboBox }

procedure TWyColorComboBox.Change;
var
  cf:TColorDialog;
begin
  if ItemIndex = 41 then begin
    try
      cf := TColorDialog.Create (nil);
      if cf.Execute then
        FCustomColor := cf.Color
      else
        FCustomColor := clBlack;
    finally
      if cf <>nil then cf.Free;
    end;
  end;
  inherited;
end;

constructor TWyColorComboBox.Create(AOwner: TComponent);
var
  i:integer;
begin
  inherited Create(AOwner);
  if Owner is TWinControl then Parent := (Owner as TWinControl);
  Sorted:= False;
  Style:= csOwnerDrawFixed;
  Items.Clear;
  for i := 0 to 40 do begin
    Items.Add(ColorToString(Colors[i].Value));
  end;
  Items.Add('Custom');
  ItemIndex := 0;
end;

destructor TWyColorComboBox.Destroy;
begin
  inherited Destroy;
end;

procedure TWyColorComboBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
var
  r,r1:TRect;
  i:integer;
begin
  if FNoText then
    with Canvas do
    begin
      FillRect(Rect);
      Brush.Color := Colors[Index].Value;
      Rectangle(Rect);
    end
  else
    with Canvas do
    begin
      FillRect(Rect);
      Brush.Color := Colors[Index].Value;
      if Index <> 41 then
        Rectangle(Rect.Left + 2,Rect.Top + 2,60,Rect.Bottom - 2)
      else begin
        for i := 2 to 60 do begin
          Brush.Color := RGB(255,255,i*4);
          r1.Left := i;
          r1.Top := Rect.Top + 2;
          r1.Right := i+1;
          r1.Bottom := Rect.Bottom - 2;
          FillRect(r1);
        end;
      end;
      r.Left := Rect.Left+64;
      r.Top := Rect.Top;
      r.Right := Rect.Right;
      r.Bottom := Rect.Bottom;
      SetBkMode(Canvas.Handle ,windows.TRANSPARENT);
      if Index <> 41 then
        DrawTextEx(Canvas.Handle,PChar(ColorToString(Colors[Index].Value)),-1,
              r,DT_VCenter or DT_SingleLine,nil)
      else
        DrawTextEx(Canvas.Handle,PChar('Custom'),-1,
              r,DT_VCenter or DT_SingleLine,nil)
    end
end;

function TWyColorComboBox.GetSelectedColor: TColor;
begin
  if ItemIndex <> 41 then
    Result := StringToColor(Items[ItemIndex])
  else
    Result := FCustomColor;
end;

function TWyColorComboBox.GetSelectedColorName: String;
begin
  Result := Items[ItemIndex];
end;

procedure TWyColorComboBox.SetNoText(const Value: boolean);
begin
  FNoText := Value;
end;

end.
