unit WySysInfo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  extctrls,Registry,AboutClass;

type
  TWySysInfo = class(TComponent)
  private
    { Private declarations }
    FAbout :TMyAboutBoxObject;
    function GetPCName: String;
    procedure SetPCName(const Value: String);
    function GetCurDir: String;
    function GetWinDir: String;
    function GetCurrUserName: String;
    function GetAvailableMemory: String;
    function GetRasDispX: String;
    function GetRasDispY: String;
    function GetTotalMemory: String;
    //read registry info
    function GetWindowsInfo(keyname:String): string;
    //write registry info
    procedure SetWindowsInfo(keyname:String;value:string);
    function GetProductId: String;
    function GetProductKey: String;
    function GetProductName: String;
    function GetProgramFilesDir: String;
    function GetRegisteredOrganization: String;
    function GetRegisteredOwner: String;
    function GetSystemRoot: String;
    function GetVersion: String;
    function GetVersionNumber: String;
    function GetWallPaperDir: String;
    procedure SetRegisteredOrganization(const Value: String);
    procedure SetRegisteredOwner(const Value: String);
    procedure SetVersion(const Value: String);
    procedure SetVersionNumber(const Value: String);
    procedure SetWallPaperDir(const Value: String);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
  published
    { Published declarations }
    property About:TMyAboutBoxObject
      read FAbout
      write FAbout;
    property AvailableMemory:String read GetAvailableMemory;
    property TotalMemory:String read GetTotalMemory;
    property ComputerName:String read GetPCName write SetPCName;
    property CurrUserName:String read GetCurrUserName;
    property WindowsDir:String read GetWinDir;
    property CurrentDir:String read GetCurDir;
    property RasDispX:String read GetRasDispX;
    property RasDispY:String read GetRasDispY;
    property ProductId:String read GetProductId;
    property ProductKey:String read GetProductKey;
    property ProductName:String read GetProductName;
    property ProgramFilesDir:String read GetProgramFilesDir;
    property RegisteredOwner:String
             read GetRegisteredOwner
             write SetRegisteredOwner;
    property RegisteredOrganization:String
             read GetRegisteredOrganization
             write SetRegisteredOrganization;
    property SystemRoot:String
             read GetSystemRoot;
    property Version:String
             read GetVersion
             write SetVersion;
    property VersionNumber:String
             read GetVersionNumber
             write SetVersionNumber;
    property WallPaperDir:String
             read GetWallPaperDir
             write SetWallPaperDir;
  end;


implementation
{$R WySysInfo.res}



{ TWySysInfo }

constructor TWySysInfo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TWySysInfo.Destroy;
begin
  inherited Destroy;
end;

function TWySysInfo.GetAvailableMemory: String;
var
  ms:TMEMORYSTATUS;
begin
  ms.dwLength := sizeof(TMEMORYSTATUS);
  GlobalMemoryStatus(ms);
  Result := IntToStr(ms.dwAvailPhys div 1024);
end;

function TWySysInfo.GetCurDir: String;
begin
  Result := GetCurrentDir;
end;

function TWySysInfo.GetCurrUserName: String;
var
  un:Array[0..255] of char;
  us:DWord;
begin
  us := 255;
  GetUserName(un,us);
  Result := StrPas(un);
end;

function TWySysInfo.GetPCName: String;
var
  cname:PChar;
  csize:^DWORD;
begin
  GetMem(cname,255);
  New(csize);
  csize^ := 255;
  if GetComputerName(cname,csize^) then
    Result := StrPas(cname)
  else
  begin
    Result := '';
  end;
  FreeMem(cname,255);
  Dispose(csize);
end;

function TWySysInfo.GetRasDispX: String;
begin
  Result := IntToStr(GetSystemMetrics(SM_CXSCREEN));
end;

function TWySysInfo.GetRasDispY: String;
begin
  Result := IntToStr(GetSystemMetrics(SM_CYSCREEN));
end;

function TWySysInfo.GetTotalMemory: String;
var
  ms:TMEMORYSTATUS;
begin
  ms.dwLength := sizeof(TMEMORYSTATUS);
  GlobalMemoryStatus(ms);
  Result := IntToStr(ms.dwTotalPhys div 1024);
end;

function TWySysInfo.GetWinDir: String;
var
  windir:Array[0..255] of Char;
begin
  GetWindowsDirectory(windir,sizeof(windir));
  Result := StrPas(windir);
end;

procedure TWySysInfo.SetPCName(const Value: String);
begin
  SetComputerName(PChar(Value));
  //Application.MessageBox(PChar('Ҫʹ޸Ч,'),PChar('ʾ'),MB_OK);
end;

function TWySysInfo.GetWindowsInfo(keyname:String): string;
var
  reg: TRegistry;
begin
  Result := '';
  reg := TRegistry.Create;
  with reg do
  begin
    RootKey := HKEY_LOCAL_MACHINE;
    OpenKey('Software\Microsoft\Windows\CurrentVersion', False);
    Result := ReadString(keyname);
  end;
  reg.Free;
end;


function TWySysInfo.GetProductId: String;
begin
  Result := GetWindowsInfo('ProductId');
end;

function TWySysInfo.GetProductKey: String;
begin
  Result := GetWindowsInfo('ProductKey');
end;

function TWySysInfo.GetProductName: String;
begin
  Result := GetWindowsInfo('ProductName');
end;

function TWySysInfo.GetProgramFilesDir: String;
begin
  Result := GetWindowsInfo('ProgramFilesDir');
end;

function TWySysInfo.GetRegisteredOrganization: String;
begin
  Result := GetWindowsInfo('RegisteredOrganization');
end;

function TWySysInfo.GetRegisteredOwner: String;
begin
  Result := GetWindowsInfo('RegisteredOwner');
end;

function TWySysInfo.GetSystemRoot: String;
begin
  Result := GetWindowsInfo('SystemRoot');
end;

function TWySysInfo.GetVersion: String;
begin
  Result := GetWindowsInfo('Version');
end;

function TWySysInfo.GetVersionNumber: String;
begin
  Result := GetWindowsInfo('VersionNumber');
end;

function TWySysInfo.GetWallPaperDir: String;
begin
  Result := GetWindowsInfo('WallPaperDir');
end;

procedure TWySysInfo.SetRegisteredOrganization(const Value: String);
begin
  SetWindowsInfo('RegisteredOrganization',Value);
end;

procedure TWySysInfo.SetRegisteredOwner(const Value: String);
begin
  SetWindowsInfo('RegisteredOwner',Value);
end;

procedure TWySysInfo.SetVersion(const Value: String);
begin
  SetWindowsInfo('Version',Value);
end;

procedure TWySysInfo.SetVersionNumber(const Value: String);
begin
  SetWindowsInfo('VersionNumber',Value);
end;

procedure TWySysInfo.SetWallPaperDir(const Value: String);
begin
  SetWindowsInfo('WallPaperDir',Value);
end;

procedure TWySysInfo.SetWindowsInfo(keyname: String;value:string);
var
  reg: TRegistry;
begin
  reg := TRegistry.Create;
  with reg do
  begin
    RootKey := HKEY_LOCAL_MACHINE;
    OpenKey('Software\Microsoft\Windows\CurrentVersion', False);
    WriteString(keyname,value);
  end;
  reg.Free;
end;

end.
