unit WyTrayIcon;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  AboutClass,ShellAPI,Menus,ExtCtrls;

const
  wm_TrayIconMessage = wm_User + 100;

type
  TWyTrayIcon = class(TComponent)
  private
    { Private declarations }
    FAbout :TMyAboutBoxObject;
    FEnabled: Boolean;
    FIconIndex: Integer;
    FHint: string;
    FIcon: TIcon;
    FIconList: TImageList;
    FIconData: TNotifyIconData;
    FPopupMenu: TPopupMenu;
    FTimer: TTimer;
    FMsgWindow: HWnd;
    function GetAnimate: Boolean;
    function GetInterval: Integer;
    procedure SetAnimate(const Value: Boolean);
    procedure SetEnabled(const Value: Boolean);
    procedure SetHint(const Value: string);
    procedure SetIconIndex(const Value: Integer);
    procedure SetInterval(const Value: Integer);
    procedure TimerHandler( Sender: TObject );

    procedure RButtonDown;
    procedure LButtonDblClk;

    procedure MinimizeAppHandler( Sender: TObject );
    procedure RestoreAppHandler( Sender: TObject );

  protected
    { Protected declarations }
    procedure Loaded; override;
    procedure Notification( Component: TComponent;
                            Operation: TOperation ); override;
    procedure MsgWndProc( var Msg: TMessage );
    procedure Update;
    property IconData: TNotifyIconData read FIconData;
  public
    { Public declarations }
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
    procedure ShowMenu;
  published
    { Published declarations }
    property About:TMyAboutBoxObject
      read FAbout
      write FAbout;

    property Animate: Boolean
      read GetAnimate
      write SetAnimate
      default False;

    property Enabled: Boolean
      read FEnabled
      write SetEnabled
      default True;

    property Hint: string
      read FHint
      write SetHint;

    property PopupMenu: TPopupMenu
      read FPopupMenu
      write FPopupMenu;

    property Icons: TImageList
      read FIconList
      write FIconList;

    property IconIndex: Integer
      read FIconIndex
      write SetIconIndex
      default 0;

    property Interval: Integer
      read GetInterval
      write SetInterval
      default 1000;
  end;


implementation
{$R WyTrayIcon.res}



{ TWyTrayIcon }

constructor TWyTrayIcon.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIcon := TIcon.Create;
  FTimer := TTimer.Create( nil );
  FIconIndex := 0;
  FIcon.Assign( Application.Icon );
  FMsgWindow := AllocateHWnd( MsgWndProc );
  FTimer.Enabled := False;
  FTimer.OnTimer := TimerHandler;
  FTimer.Interval := 1000;
  if not ( csDesigning in ComponentState ) then
  begin

    FillChar( FIConData, 0, SizeOf( TNotifyIconData ) );
    with FIconData do
    begin
      cbSize := SizeOf( TNotifyIconData );
      Wnd := FMsgWindow;
      uID := Integer( Self );
      uFlags := nif_Message or nif_Icon;
      uCallBackMessage := wm_TrayIconMessage;
      hIcon := FIcon.Handle;
      szTip := '';
    end;

    Application.OnMinimize := MinimizeAppHandler;
    Application.OnRestore := RestoreAppHandler;
    Application.OnDeactivate := MinimizeAppHandler;

    Update;
  end;
  Enabled := True;
end;

destructor TWyTrayIcon.Destroy;
begin
  if not ( csDesigning in ComponentState ) then
    Shell_NotifyIcon( nim_Delete, @FIconData );
  DeallocateHWnd( FMsgWindow );
  FIcon.Free;
  FTimer.Free;
  inherited;
end;

function TWyTrayIcon.GetAnimate: Boolean;
begin
  Result := FTimer.Enabled;
end;

function TWyTrayIcon.GetInterval: Integer;
begin
  Result := FTimer.Interval;
end;

procedure TWyTrayIcon.LButtonDblClk;
begin
  RestoreAppHandler(self);
end;

procedure TWyTrayIcon.Loaded;
begin
  inherited Loaded;
  if not Assigned( FIconList ) then
    FIcon.Assign( Application.Icon )
  else
    FIconList.GetIcon( FIconIndex, FIcon );
  Update;
end;

procedure TWyTrayIcon.MinimizeAppHandler( Sender: TObject );
begin
  Application.Minimize;
  if FEnabled then
    ShowWindow( Application.Handle, sw_Hide );
end;

procedure TWyTrayIcon.MsgWndProc(var Msg: TMessage);
begin
  with Msg do
  begin
    case Msg of
      wm_TrayIconMessage:
      begin
        case LParam of
          WM_RButtonDown:RButtonDown;
          WM_LBUTTONDBLCLK:LButtonDblClk;
        end;
      end;
    end;
  end;
  Dispatch( Msg );
end;

procedure TWyTrayIcon.Notification(Component: TComponent;
  Operation: TOperation);
begin
  inherited Notification( Component, Operation );
  if Operation = opRemove then
  begin
    if Component = FPopupMenu then
      FPopupMenu := nil
    else if Component = FIconList then
      FIconList := nil;
  end;
end;

procedure TWyTrayIcon.RButtonDown;
begin
  ShowMenu;
end;

procedure TWyTrayIcon.RestoreAppHandler( Sender: TObject );
begin
  Application.Restore;
  if FEnabled then
    ShowWindow( Application.Handle, sw_Show );
  SetForeGroundWindow(Application.Handle);
end;

procedure TWyTrayIcon.SetAnimate(const Value: Boolean);
begin
  FTimer.Enabled := Value;
end;

procedure TWyTrayIcon.SetEnabled(const Value: Boolean);
begin
  FEnabled := Value;
  if not ( csDesigning in ComponentState ) then
  begin
    if FEnabled then
    begin
      if not Shell_NotifyIcon( nim_Add, @FIconData ) then
        Application.MessageBox(pchar('Can not Create Task Icon'),pchar('Error'),MB_OK);
    end
    else
    begin
      if not Shell_NotifyIcon( nim_Delete, @FIconData ) then
        Application.MessageBox(pchar('Can not Remove Task Icon'),pchar('Error'),MB_OK);
    end;
  end;
end;

procedure TWyTrayIcon.SetHint(const Value: string);
begin
  if ( FHint <> Value ) and ( Length( Value ) < 64 ) then
  begin
    FHint := Value;
    StrPLCopy( FIconData.szTip, Value, SizeOf( FIconData.szTip ) - 1 );

    if Value <> '' then
      FIconData.uFlags := FIconData.uFlags or nif_Tip
    else
      FIconData.uFlags := FIconData.uFlags and not nif_Tip;

    Update;
  end;
end;

procedure TWyTrayIcon.SetIconIndex(const Value: Integer);
begin
  FIconIndex := Value;
  if Assigned( FIconList ) then
    FIconList.GetIcon( FIconIndex, FIcon );
  Update;
end;

procedure TWyTrayIcon.SetInterval(const Value: Integer);
begin
  FTimer.Interval := Value;
end;

procedure TWyTrayIcon.ShowMenu;
var
  P: TPoint;
begin
  if Assigned( FPopupMenu ) then
  begin
    GetCursorPos( P );
    if Owner is TForm then
      SetForegroundWindow( TForm( Owner ).Handle );
    FPopupMenu.Popup( P.X, P.Y );
    Application.ProcessMessages;
  end;
end;


procedure TWyTrayIcon.TimerHandler(Sender: TObject);
begin
  if FIconList <> nil then
  begin
    if IconIndex < FIconList.Count - 1 then
      Inc( FIconIndex )
    else
      FIconIndex := 0;
    SetIconIndex( FIconIndex );
    Update;
  end;
end;

procedure TWyTrayIcon.Update;
begin
  if not ( csDesigning in ComponentState ) then
  begin
    FIconData.hIcon := FIcon.Handle;
    Shell_NotifyIcon( nim_Modify, @FIconData );
  end;
end;

end.
 