unit LabelEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TLabelDirection = (ldUp,ldLeft,ldDown,ldRight);
  TLabelEdit = class(TEdit)
  private
    FLabel:TLabel;
    FLabelCaption: string;
    FLabelDirection: TLabelDirection;
    FLabelFont: TFont;
    procedure SetLabelCaption(const Value: string);
    procedure wmsize(var Message:TWMSIZE); Message WM_SIZE;
    procedure wmmove(var Message:TWMMOVE); Message WM_MOVE;
    procedure wmpaint(var Message:TWMPAINT); Message WM_PAINT;
    procedure SetLabelDirection(const Value: TLabelDirection);
    procedure SetLabelFont(const Value: TFont);
    procedure CMTextChanged(var Message:TMessage);message CM_TEXTCHANGED;
    procedure WMSetFont(var Message: TWMSetFont); message WM_SETFONT;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
  published
    { Published declarations }
    property LabelCaption:string read FLabelCaption write SetLabelCaption;
    property LabelDirection:TLabelDirection read FLabelDirection write SetLabelDirection;
    property LabelFont:TFont read FLabelFont write SetLabelFont;
  end;


implementation

{ TLabelEdit }
procedure TLabelEdit.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TLabelEdit.CMTextChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

constructor TLabelEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLabel := TLabel.Create(self);
  FLabel.Parent := TWinControl(Owner);
  FLabel.Visible := True;
  FLabel.Caption := 'Caption';
  FLabelCaption := 'Caption';
  FLabel.Height := Height;
  FLabel.Width := Width;
  FLabel.Left := Left;
  FLabel.Top := Top - FLabel.Height;
  FLabelFont := TFont.Create;
  FLabelFont.Assign(Font);
end;

destructor TLabelEdit.Destroy;
begin
  FLabel.Free;
  FLabelFont.Free;
  inherited Destroy;
end;

procedure TLabelEdit.SetLabelCaption(const Value: string);
begin
  FLabelCaption := Value;
  FLabel.Caption := Value;
  invalidate;
end;

procedure TLabelEdit.SetLabelDirection(const Value: TLabelDirection);
begin
  FLabelDirection := Value;
  Invalidate;
end;

procedure TLabelEdit.SetLabelFont(const Value: TFont);
begin
  FLabelFont.assign(Value);
  FLabel.Font.Assign(FLabelFont);
  FLabel.Invalidate;
  Invalidate;
end;

procedure TLabelEdit.wmmove(var Message: TWMMOVE);
begin
  inherited;
  Invalidate;
end;

procedure TLabelEdit.wmpaint(var Message: TWMPAINT);
begin
  inherited;
  case FLabelDirection of
    ldUp:
      begin
        FLabel.Left := Left;
        FLabel.Top := Top - FLabel.Height - 2;
      end;
    ldLeft:
      begin
        FLabel.Left := Left - FLabel.Width - 2;
        FLabel.Top := Top;
      end;
    ldDown:
      begin
        FLabel.Left := Left;
        FLabel.Top := Top + Height + 2;
      end;
    ldRight:
      begin
        FLabel.Left := Left + Width + 2;
        FLabel.Top := Top;
      end;
  end;
  FLabel.Font := FLabelFont;
end;

procedure TLabelEdit.WMSetFont(var Message: TWMSetFont);
begin
  inherited;
  Invalidate;
end;

procedure TLabelEdit.wmsize(var Message: TWMSIZE);
begin
  inherited;
  Invalidate;
end;

end.
