unit TimerLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,Extctrls;

type
  TBorderKind=(bkNone,bkRaised,bkETCHED,bkBump,bkSUNKEN);
  TTimerLabel = class(TLabel)
  private
    { Private declarations }
    FTimer : TTimer;
    FFlashColor : boolean;
    FBorderKind: TBorderKind;
    procedure DoTimer(Sender: TObject);
    procedure SetFlashColor(Value:boolean);
    procedure SetBorderKind(Value:TBorderKind);
    procedure EraseBkGround(var Message:TWMERASEBKGND);message WM_ERASEBKGND;
  protected
    { Protected declarations }
    procedure Paint;override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
  published
    { Published declarations }
    property FlashColor :boolean read FFlashColor write SetFlashColor;
    property BorderKind:TBorderKind read FBorderKind write SetBorderKind;
  end;


implementation

{ TTimerLabel }

constructor TTimerLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Transparent := True;
  AutoSize := False;
  Alignment := taCenter;
  FTimer := TTimer.Create( nil );
  FTimer.Interval := 1000;
  FTimer.OnTimer := DoTimer;
end;

destructor TTimerLabel.Destroy;
begin
  FTimer.Free;
  inherited Destroy;
end;

procedure TTimerLabel.DoTimer;
var
  r:TRect;
begin
  inherited;
  r := GetClientRect;
  if FFlashColor then begin
    Randomize;
    Font.Color := RGB(Random(255),Random(255),Random(255));
  end;
  case FBorderKind of
    bkRaised:DrawEdge(Canvas.Handle,r,EDGE_RAISED,BF_RECT);
    bkEtched:DrawEdge(Canvas.Handle,r,EDGE_ETCHED,BF_RECT);
    bkBump:DrawEdge(Canvas.Handle,r,EDGE_BUMP,BF_RECT);
    bkSunken:DrawEdge(Canvas.Handle,r,EDGE_SUNKEN,BF_RECT);
  end;
  Caption := DateTimeToStr(now);
end;

procedure TTimerLabel.EraseBkGround(var Message: TWMERASEBKGND);
begin
  Message.Result := 0;
end;

procedure TTimerLabel.Paint;
begin
  inherited;
  DoTimer(self);
end;

procedure TTimerLabel.SetBorderKind(Value: TBorderKind);
begin
  if FBorderKind <> Value then begin
    FBorderKind := Value;
    Invalidate;
  end;
end;

procedure TTimerLabel.SetFlashColor(Value: boolean);
begin
  if FlashColor <> Value then begin
    FFlashColor := Value;
    Invalidate;
  end;
end;

end.
