unit WYGrid;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids;

type
  {TWYGrid}
  TWYGrid = class(TStringGrid)
  private
    { Private declarations }
    FTwoColors:boolean;
    FColor1: TColor;
    FColor2: TColor;
    FEnterToTab: boolean;
    procedure SetTwoColors(Value:boolean);
    procedure SetColor1(Value:TColor);
    procedure SetColor2(Value:TColor);
    procedure SetEnterToTab(const Value: boolean);
    procedure WMKeyPress(var Message:TWMKey);message WM_KEYDOWN;
  protected
    { Protected declarations }
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect;
              AState: TGridDrawState);override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    procedure Paint;override;
  published
    { Published declarations }
    property TwoColors :boolean read FTwoColors write SetTwoColors default True;
    property FirstColor:TColor read FColor1 write SetColor1 default clGreen;
    property SecondColor:TColor read FColor2 write SetColor2 default clInfoBk;
    property EnterToTab:boolean read FEnterToTab write SetEnterToTab;
  end;

implementation

{ TWYGrid }

constructor TWYGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FColor1 := $00D1A3A3;
  FColor2 := clInfoBk;
  FTwoColors := True;
  DefaultRowHeight := 20;
  Height := 109;
  Width := 329;
end;


procedure TWYGrid.Paint;
var
  i,j:integer;
  r:TRect;
begin
  inherited;
  if not FTwoColors then Exit;
  for i := FixedRows to RowCount - 1 do begin
    if odd(i) then begin
      for j := FixedCols to ColCount - 1 do begin
        r := CellRect(j,i);
        Canvas.Brush.Color := FColor1;
        Canvas.FillRect(r);
        DrawCell(j,i,r,[gdSelected, gdFocused, gdFixed]);
      end;
    end
    else
    begin
      for j := FixedCols to ColCount - 1 do begin
        r := CellRect(j,i);
        Canvas.Brush.Color := FColor2;
        Canvas.FillRect(r);
        DrawCell(j,i,r,[gdSelected, gdFocused, gdFixed]);
      end;
    end;
  end;
  Canvas.DrawFocusRect(CellRect(Col,Row));
end;

procedure TWYGrid.SetColor1(Value: TColor);
begin
  if FColor1 <> Value then FColor1 := Value;
  Invalidate;
end;

procedure TWYGrid.SetColor2(Value: TColor);
begin
  if FColor2 <> Value then FColor2 := Value;
  Invalidate;
end;

procedure TWYGrid.SetTwoColors(Value: boolean);
begin
  if FTwoColors <> Value then FTwoColors := Value;
  Invalidate;
end;

procedure TWYGrid.DrawCell(ACol, ARow: Longint; ARect: TRect;
  AState: TGridDrawState);
begin
  if DefaultDrawing then
    Canvas.TextRect(ARect, ARect.Left+2, ARect.Top+2, Cells[ACol, ARow]);
  inherited DrawCell(ACol, ARow, ARect, AState);
end;


procedure TWYGrid.SetEnterToTab(const Value: boolean);
begin
  FEnterToTab := Value;
end;

procedure TWYGrid.WMKeyPress(var Message: TWMKey);
begin
  if (Message.CharCode = 13) and (FEnterToTab) then begin
    if ColCount <> col + 1 then col := col + 1 else
      if RowCount <> Row + 1 then begin
        Col := FixedCols;
        Row := Row + 1;
      end;
  end;
  Message.Result := 1;
end;


end.
