unit WYHint;

interface

uses
  Windows, Classes, Messages, SysUtils, Forms, Graphics, Controls;

type
  THintShape = (hsRectangle,hsRoundRect,hsElliptic);
  TWYHint = class(TComponent)
  private
    FColor         : TColor;
    FFont          : TFont;
    FPauseTime     : Integer;
    FHidePauseTime : Integer;
    FTransparent   : Boolean;
    FHintShapeA    : THintShape;
    FPenColorA : TColor;
    FPenWidthA : Integer;
    procedure SetHidePauseTime(Value: Integer);
    procedure SetPauseTime(Value: Integer);
    procedure SetHintShape(Value: THintShape);
    procedure SetPenColor(Value:TColor);
    procedure SetPenWidth(Value:Integer);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Color: TColor read FColor write FColor default clYellow;
    property Font: TFont read FFont write FFont;
    property HidePauseTime: Integer read FHidePauseTime write SetHidePauseTime default 2500;
    property PauseTime: Integer read FPauseTime write SetPauseTime default 500;
    property Transparent: Boolean read FTransparent write FTransparent default false;
    property HintShape:THintShape read FHintShapeA write SetHintShape default hsRectangle;
    property PenColor :TColor read FPenColorA write SetPenColor default clBlack;
    property PenWidth :Integer read FPenWidthA write SetPenWidth default 1;
  end;
  TWYHintWindow = class(THintWindow)
  private
    procedure WMEraseBKGND(var Message: TWMEraseBKGND); message WM_ERASEBKGND;
  protected
    procedure Paint; override;
  public
    procedure CreateParams(var Params: TCreateParams); override;
    constructor Create(AOwner: TComponent); override;
    function CalcHintRect(MaxWidth: Integer; const AHint: string; AData: Pointer): TRect; override;
  end;

implementation
{$R WyHint.res}


var
  TheWYHint : TWYHint = nil;
  FHintShape : THintShape;
  FPenColor : TColor;
  FPenWidth : Integer;

constructor TWYHint.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if TheWYHint<>nil then
    raise Exception.Create('Cannot create the TWYHint component more than one!');
  FColor := $00F0BFC7;
  FHidePauseTime := 2500;
  FPauseTime := 500;
  FTransparent := false;
  FHintShapeA := hsRectangle;
  FHintShape := hsRectangle;
  FPenColor := clBlack;
  FPenWidth := 1;
  FFont := TFont.Create;
  with FFont do begin
    Name := 'MS Sans Serif';
    Size := 10;
    Color := clNavy;
  end;
  TheWYHint := Self;
  if not (csDesigning in ComponentState) then begin
    HintWindowClass := TWYHintWindow;
    with Application do begin
      ShowHint := not ShowHint;
      ShowHint := not ShowHint;
      HintPause := FPauseTime;
      HintHidePause := FHidePauseTime;
    end;
  end;
end;

destructor TWYHint.Destroy;
begin
  FFont.Free;
  if TheWYHint=Self then begin
    if not (csDesigning in ComponentState) then begin
      HintWindowClass := THintWindow;
      with Application do begin
        ShowHint := not ShowHint;
        ShowHint := not ShowHint;
        HintPause := 500;
        HintHidePause := 2500;
      end;
    end;
    TheWYHint := nil;
  end;
  inherited Destroy;
end;

procedure TWYHint.SetHidePauseTime(Value: Integer);
begin
  if FHidePauseTime<>Value then begin
    if Value<0 then raise Exception.Create('Invalid property value!');
    FHidePauseTime := Value;
    if not (csDesigning in ComponentState) then
      Application.HintHidePause := FHidePauseTime;
  end;
end;

procedure TWYHint.SetHintShape(Value: THintShape);
begin
  FHintShapeA := Value;
  FHintShape := FHintShapeA;
end;

procedure TWYHint.SetPauseTime(Value: Integer);
begin
  if FPauseTime<>Value then begin
    if Value<0 then raise Exception.Create('Invalid property value!');
    FPauseTime := Value;
    if not (csDesigning in ComponentState) then
      Application.HintPause := FPauseTime;
  end;
end;

constructor TWYHintWindow.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if TheWYHint=nil then
    raise Exception.Create('TWYHint component has not been created!');
  ControlStyle := ControlStyle + [csOpaque];
end;

procedure TWYHintWindow.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do Style := Style and (not WS_BORDER);
end;

procedure TWYHintWindow.Paint;
var
  R: TRect;
  rgn:HRGN;
begin
  R := ClientRect;
  R.Left := R.Left;
  R.Top := R.Top;
  case FHintShape of
    hsRectangle : rgn := CreateRectRgn(R.Left,R.Top,R.Right,R.Bottom);
    hsRoundRect : rgn := CreateRoundRectRGN(R.Left,R.Top,R.Right,R.Bottom,R.Bottom*3 div 4,R.Right*3 div 4);
    hsElliptic  : rgn := CreateEllipticRGN(R.Left,R.Top,R.Right,R.Bottom);
  else
    Exit;
  end;
  SetWindowRgn(Handle,rgn,true);
  with Canvas do begin
    Pen.Color := FPenColor;
    Pen.Width := FPenWidth;
    Brush.Color := TheWYHint.Color;
    if TheWYHint.Transparent then
      Brush.Style := bsClear
    else Brush.Style := bsSolid;
    case FHintShape of
      hsRectangle : Rectangle(R.Left,R.Top,R.Right-3,R.Bottom-3);
      hsRoundRect : RoundRect(R.Left,R.Top,R.Right-3,R.Bottom-3,(R.Bottom-3)*3 div 4,(R.Right-3)*3 div 4);
      hsElliptic  : Ellipse(R.Left,R.Top,R.Right-3,R.Bottom-3);
    else
      Exit;
    end;
    SetBkMode(Handle,TRANSPARENT);
    R.Left := R.Left + 3;
    R.Top := R.Top + 3;
    R.Right := R.Right - 3;
    R.Bottom := R.Bottom - 3;
    DrawText(Handle, PChar(Caption), -1, R, DT_CENTER or DT_VCENTER);
  end;
end;

(**** Hintڵĳߴ ****)
function TWYHintWindow.CalcHintRect(MaxWidth: Integer; const AHint: string; AData: Pointer): TRect;
begin
  Canvas.Font.Assign(TheWYHint.Font);
  Result := inherited CalcHintRect(MaxWidth, AHint, AData);
end;

(**** WM_ERASEBKGNDϢʹWindowsԶñɫHint ****)
procedure TWYHintWindow.WMEraseBKGND(var Message:TWMEraseBKGND);
begin
  Message.Result := 0;//False
end;

procedure TWYHint.SetPenColor(Value: TColor);
begin
  FPenColorA := Value;
  FPenColor := FPenColorA;
end;

procedure TWYHint.SetPenWidth(Value: Integer);
begin
  FPenWidthA := Value;
  FPenWidth := FPenWidthA;
end;

end.

