unit WYShape;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TGradientStyle = (gsHorizontal, gsVertical, gsElliptic, gsRectangle,
                    gsVertCenter, gsHorizCenter);
  TWYShape = class(TShape)
  private
    { Private declarations }
    FBeginClr : TColor;    //Start color for gradient
    FEndClr : TColor;     //End color for gradient
    FGradientStyle : TGradientStyle;  //One of six choices
    procedure SetBeginClr(Value : TColor);  //Set start color of gradient
    procedure SetEndClr(Value : TColor);    //Set end color of gradient
    procedure SetGradient(Value : TGradientStyle);  //Choose gradient fill style
    procedure DoHorizontal(fr, fg, fb, dr, dg, db : Integer);
    procedure DoVertical(fr, fg, fb, dr, dg, db : Integer); // the gradients.
    procedure DoElliptic(fr, fg, fb, dr, dg, db : Integer); // The fr fg fb etc
    procedure DoRectangle(fr, fg, fb, dr, dg, db : Integer); // are color values.
    procedure DoVertCenter(fr, fg, fb, dr, dg, db : Integer);
    procedure DoHorizCenter(fr, fg, fb, dr, dg, db : Integer);
  protected
    { Protected declarations }
    procedure Paint;override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
  published
    { Published declarations }
    property Align;
    property Anchors;
    //property Brush: TBrush read FBrush write SetBrush;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Constraints;
    property ParentShowHint;
    //property Pen: TPen read FPen write SetPen;
    property Shape;//: TShapeType read FShape write SetShape default stRectangle;
    property ShowHint;
    property Visible;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property BeginColor : TColor read FBeginClr
       write SetBeginClr default clNavy;
    property EndColor : TColor read FEndClr
       write SetEndClr default clAqua;
    property GradientStyle : TGradientStyle read FGradientStyle
       write SetGradient default gsVertCenter;
  end;


implementation

{ TWYShape }

constructor TWYShape.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Shape := stRectangle;
  FBeginClr := $00FF6262;            //Start Color is at the Top of button
  FEndClr := clWhite;             //Stop Color is at the Bottom of button
end;

procedure TWYShape.Paint;
var
  FromR, FromG, FromB : Integer; //These are the separate color values for RGB
  DiffR, DiffG, DiffB : Integer; // of color values.
begin
  inherited;
  FromR := FBeginClr and $000000ff;  //Strip out separate RGB values
  FromG := (FBeginClr shr 8) and $000000ff;
  FromB := (FBeginClr shr 16) and $000000ff;
  DiffR := (FEndClr and $000000ff) - FromR;   //Find the difference
  DiffG := ((FEndClr shr 8) and $000000ff) - FromG;
  DiffB := ((FEndClr shr 16) and $000000ff) - FromB;
  case FGradientStyle of
    gsHorizontal  : DoHorizontal(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsVertical    : DoVertical(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsElliptic    : DoElliptic(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsRectangle   : DoRectangle(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsVertCenter  : DoVertCenter(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsHorizCenter : DoHorizCenter(FromR, FromG, FromB, DiffR, DiffG, DiffB);
  end;
end;

procedure TWYShape.DoHorizontal(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
begin
  ColorRect.Top:= 0;                //Set rectangle top
  ColorRect.Bottom := Height;
  for I := 0 to 255 do begin         //Make lines (rectangles) of color
    ColorRect.Left:= Muldiv (I, Width, 256);    //Find left for this color
    ColorRect.Right:= Muldiv (I + 1, Width, 256);   //Find Right
    R := fr + Muldiv(I, dr, 255);    //Find the RGB values
    G := fg + Muldiv(I, dg, 255);
    B := fb + Muldiv(I, db, 255);
    Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    Canvas.FillRect(ColorRect);           //Draw on Bitmap
  end;
end;

procedure TWYShape.DoVertical(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
begin
  ColorRect.Left:= 0;                //Set rectangle left&right
  ColorRect.Right:= Width;
  for I := 0 to 255 do begin         //Make lines (rectangles) of color
    ColorRect.Top:= Muldiv (I, Height, 256);    //Find top for this color
    ColorRect.Bottom:= Muldiv (I + 1, Height, 256);   //Find Bottom
    R := fr + Muldiv(I, dr, 255);    //Find the RGB values
    G := fg + Muldiv(I, dg, 255);
    B := fb + Muldiv(I, db, 255);
    Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    Canvas.FillRect(ColorRect);           //Draw on Bitmap
  end;
end;

procedure TWYShape.DoElliptic(fr, fg, fb, dr, dg, db : Integer);
var
  I: Integer;
  R, G, B : Byte;
  Pw, Ph, Dw, Dh : integer;
  x1,y1,x2,y2 : integer;
{The elliptic is a bit different, since I had to use real numbers. I cut down
 on the number (to 155 instead of 255) of iterations in an attempt to speed
 things up, to no avail.  I think it just takes longer for windows to draw an
 ellipse as opposed to a rectangle.}
begin
  Canvas.Pen.Style := psClear;
  Canvas.Pen.Mode := pmCopy;
  x1 := Width div -3;
  x2 := Width + (Width div 3);
  y1 := Height div -3;
  y2 := Height + (Height div 3);
  Pw := x2 - x1;
  Ph := y2 - y1;
  for I := 0 to 50 do begin         //Make ellipses of color
    R := fr + Muldiv(I, dr, 50);    //Find the RGB values
    G := fg + Muldiv(I, dg, 50);
    B := fb + Muldiv(I, db, 50);
    Canvas.Brush.Color := R or (G shl 8) or (b shl 16);   //Plug colors into brush
    Dw := Pw * i div 100;
    Dh := Ph * i div 100;
    Canvas.Ellipse(x1 + Dw,y1 + Dh,x2 - Dw,y2 - Dh);
  end;
  Canvas.Pen.Style := psSolid;
end;

procedure TWYShape.DoRectangle(fr, fg, fb, dr, dg, db : Integer);
var
  I: Integer;
  R, G, B : Byte;
  Pw, Ph : Real;
  x1,y1,x2,y2 : Real;
begin
  Canvas.Pen.Style := psClear;
  Canvas.Pen.Mode := pmCopy;
  x1 := 0;
  x2 := Width+2;
  y1 := 0;
  y2 := Height+2;
  Pw := (Width / 2) / 255;
  Ph := (Height / 2) / 255;
  for I := 0 to 255 do begin         //Make rectangles of color
    x1 := x1 + Pw;
    x2 := X2 - Pw;
    y1 := y1 + Ph;
    y2 := y2 - Ph;
    R := fr + Muldiv(I, dr, 255);    //Find the RGB values
    G := fg + Muldiv(I, dg, 255);
    B := fb + Muldiv(I, db, 255);
    Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    Canvas.FillRect(Rect(Trunc(x1),Trunc(y1),Trunc(x2),Trunc(y2)));
  end;
  Canvas.Pen.Style := psSolid;
end;

procedure TWYShape.DoVertCenter(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
  Haf : Integer;
begin
  Haf := Height Div 2;
  ColorRect.Left := 0;
  ColorRect.Right := Width;
  for I := 0 to Haf do begin
    ColorRect.Top := Muldiv (I, Haf, Haf);
    ColorRect.Bottom := Muldiv (I + 1, Haf, Haf);
    R := fr + Muldiv(I, dr, Haf);
    G := fg + Muldiv(I, dg, Haf);
    B := fb + Muldiv(I, db, Haf);
    Canvas.Brush.Color := RGB(R, G, B);
    Canvas.FillRect(ColorRect);
    ColorRect.Top := Height - (Muldiv (I, Haf, Haf));
    ColorRect.Bottom := Height - (Muldiv (I + 1, Haf, Haf));
    Canvas.FillRect(ColorRect);
  end;
end;

procedure TWYShape.DoHorizCenter(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
  Haf : Integer;
begin
  Haf := Width Div 2;
  ColorRect.Top := 0;
  ColorRect.Bottom := Height;
  for I := 0 to Haf do begin
    ColorRect.Left := Muldiv (I, Haf, Haf);
    ColorRect.Right := Muldiv (I + 1, Haf, Haf);
    R := fr + Muldiv(I, dr, Haf);
    G := fg + Muldiv(I, dg, Haf);
    B := fb + Muldiv(I, db, Haf);
    Canvas.Brush.Color := RGB(R, G, B);
    Canvas.FillRect(ColorRect);
    ColorRect.Left := Width - (Muldiv (I, Haf, Haf));
    ColorRect.Right := Width - (Muldiv (I + 1, Haf, Haf));
    Canvas.FillRect(ColorRect);
  end;
end;

procedure TWYShape.SetBeginClr(Value : TColor);
begin
  if FBeginClr <> Value then begin
    FBeginClr := Value;
    Invalidate;
  end;
end;

procedure TWYShape.SetEndClr(Value : TColor);
begin
  if FEndClr <> Value then begin
    FEndClr := Value;
    Invalidate;
  end;
end;


procedure TWYShape.SetGradient(Value: TGradientStyle);
begin
  If FGradientStyle <> Value then begin
    FGradientStyle := Value;
    Invalidate;
  end;
end;

end.
