unit WyCustomControl;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls,Graphics;

type
  TWyMouseState = (msDown,msUp,msMove,msLeave);
  TWyCustomControl = class(TCustomControl)
  private
    { Private declarations }
    MouseState : TWyMouseState;
    FOnMouseLeave: TNotifyEvent;
    FMouseMoveColor: TColor;
    FNormalFrameColor: TColor;
    FNormalColor: TColor;
    FMouseDownColor: TColor;
    procedure WyDrawFocus;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure WMKeyDown (var Message: TWMKeyDown); message WM_KEYDOWN;
    procedure WMKeyUp (var Message: TWMKeyUp); message WM_KEYUP;
    procedure SetMouseDownColor(const Value: TColor);
    procedure SetMouseMoveColor(const Value: TColor);
    procedure SetNormalColor(const Value: TColor);
    procedure SetNormalFrameColor(const Value: TColor);
  protected
    { Protected declarations }
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer);override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);override;
  published
    { Published declarations }
    property Caption;
    property Font;
    property OnMouseLeave:TNotifyEvent
             read FOnMouseLeave
             write FOnMouseLeave;
    property NormalColor:TColor read FNormalColor write SetNormalColor;
    property NormalFrameColor:TColor read FNormalFrameColor write SetNormalFrameColor;
    property MouseMoveColor:TColor read FMouseMoveColor write SetMouseMoveColor;
    property MouseDownColor:TColor read FMouseDownColor write SetMouseDownColor;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelKind;
    property BevelWidth;
    property BorderWidth;
    property Enabled;
    property Ctl3D;
    property ImeMode;
    property ImeName;
    property ParentCtl3D;
    property TabOrder;
    property TabStop;
    property Hint;
    property ShowHint;
    property OnDockDrop;
    property OnDockOver;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnUnDock;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
  end;


implementation

{ TWyCustomControl }

procedure TWyCustomControl.CMFontChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TWyCustomControl.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  MouseState := msMove;
  Invalidate;
end;

procedure TWyCustomControl.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  MouseState := msLeave;
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
  Invalidate;
end;

procedure TWyCustomControl.CMTextChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

constructor TWyCustomControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 75;
  Height := 25;
  MouseState := msLeave;
  FNormalColor := $00E1EAEB;
  FNormalFrameColor := $00839600;
  FMouseMoveColor := $00B4CDCC;
  FMouseDownColor := $00C5D6D9;
end;

destructor TWyCustomControl.Destroy;
begin
  inherited Destroy;
end;


procedure TWyCustomControl.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  if Button <> mbLeft then Exit;
  MouseState := msDown;
  Invalidate;
end;

procedure TWyCustomControl.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if MouseState = msDown then Exit;
  inherited;
  MouseState := msMove;
  Invalidate;
end;

procedure TWyCustomControl.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  MouseState := msUp;
  Invalidate;
end;

procedure TWyCustomControl.Paint;
begin
  inherited Paint;
  if Enabled then WyDrawFocus else begin
    Canvas.Font.Assign(Font);
    Canvas.Pen.Color := FNormalFrameColor;
    Canvas.Brush.Color := FNormalColor;
    Canvas.FillRect(ClientRect);
    Canvas.Rectangle(ClientRect);
    Canvas.TextOut((Width - Canvas.TextWidth(Caption)) div 2,
                   (Height - Canvas.TextHeight(Caption)) div 2,
                    Caption);
  end;
  if Focused then
    Canvas.DrawFocusRect(Rect(ClientRect.Left + 2,ClientRect.Top + 2,ClientRect.Right - 2,ClientRect.Bottom - 2));
end;

procedure TWyCustomControl.SetMouseDownColor(const Value: TColor);
begin
  FMouseDownColor := Value;
  Invalidate;
end;

procedure TWyCustomControl.SetMouseMoveColor(const Value: TColor);
begin
  FMouseMoveColor := Value;
  Invalidate;
end;

procedure TWyCustomControl.SetNormalColor(const Value: TColor);
begin
  FNormalColor := Value;
  Invalidate;
end;

procedure TWyCustomControl.SetNormalFrameColor(const Value: TColor);
begin
  FNormalFrameColor := Value;
  Invalidate;
end;

procedure TWyCustomControl.WMKeyDown(var Message: TWMKeyDown);
begin
  if Message.CharCode = VK_RETURN then
  begin
    MouseState := msDown;
    Invalidate;
  end
  else
    inherited;
end;

procedure TWyCustomControl.WMKeyUp(var Message: TWMKeyUp);
begin
  inherited;
  MouseState := msUp;
  Invalidate;
end;     

procedure TWyCustomControl.WyDrawFocus;
var
  rect:TRect;
begin
  rect := ClientRect;
  Canvas.Font.Assign(Font);
  case MouseState of
    msMove:
      begin
        Canvas.Brush.Color := FMouseMoveColor;
        Canvas.FillRect(rect);
        Canvas.Rectangle(rect);
        DrawEdge(Canvas.Handle,rect,BDR_RAISEDINNER,BF_RECT);
        Canvas.TextOut((Width - Canvas.TextWidth(Caption)) div 2 - 1,
                       (Height - Canvas.TextHeight(Caption)) div 2 - 1,
                        Caption);
      end;
    msLeave:
      begin
        Canvas.Pen.Color := FNormalFrameColor;
        Canvas.Brush.Color := FNormalColor;
        Canvas.FillRect(rect);
        Canvas.Rectangle(rect);
        Canvas.TextOut((Width - Canvas.TextWidth(Caption)) div 2,
                       (Height - Canvas.TextHeight(Caption)) div 2,
                        Caption);
     end;
    msDown:
      begin
        Canvas.Brush.Color := FMouseDownColor;
        Canvas.FillRect(rect);
        Canvas.Rectangle(rect);
        DrawEdge(Canvas.Handle,rect,BDR_SUNKENINNER,BF_RECT);
        Canvas.TextOut((Width - Canvas.TextWidth(Caption)) div 2 + 1,
                       (Height - Canvas.TextHeight(Caption)) div 2+ 1,
                        Caption);
      end;
    msUp:
      begin
        Canvas.Brush.Color := FNormalColor;
        Canvas.FillRect(rect);
        Canvas.Rectangle(rect);
        DrawEdge(Canvas.Handle,rect,BDR_RAISEDINNER,BF_RECT);
        Canvas.TextOut((Width - Canvas.TextWidth(Caption)) div 2 - 1,
                       (Height - Canvas.TextHeight(Caption)) div 2 - 1,
                        Caption);
      end;
  end;
end;

end.
